/****************************************************************************
                             Hyper's CD Catalog
		A multiplatform qt and xml based catalog program

 Author    : Peter Deak (hyperr@freemail.hu)
 License   : GPL
 Copyright : (C) 2003 Peter Deak
****************************************************************************/

#include "config.h"

#include <qstring.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qvariant.h>
#include <qcheckbox.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qwhatsthis.h>
#include <qfile.h>
#include <qframe.h>
#include <qtextstream.h>


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <iostream>

#include "dbase.h"
#include "adddialog.h"
#include "mainwidget.h"

#ifdef _WIN32
#define CONFIGFILE "cdcat.cfg"
#else
#define CONFIGFILE ".cdcatconf"
#endif

CdCatConfig::CdCatConfig(void)
 {
  fsize      = 8;
  autoload   = false;
  autoloadfn = "";
  nice       = true;
  cdrompath  ="";
#ifndef _WIN32
  mounteject = true;
#endif
  }

int CdCatConfig::startProgram(DataBase **dbp,QWidget *mw)
 {
  if(autoload)
   {
    if((*dbp) == NULL)
     (*dbp) = new DataBase();

    PWw *pww = new PWw(mw);
    (*dbp)->pww = pww;
    progress(pww);
    if((*dbp)->openDB((char *)((const char *)autoloadfn)))
     {
      (*dbp) = NULL;
      QMessageBox::warning(mw,tr("Error during autoload..."),
       tr("I can't open the autoload catalog.\nCheck the file, or change the autoload option in the config dialog!\n"),
      tr("Ok"),0);
     }
    progress(pww);
    pww->end();
    if((*dbp) != NULL)
       (*dbp)->pww = NULL;
    delete pww;
   }
  return 0;
 }

int CdCatConfig::readConfig(void)
 {
  FILE *cf;
  char str[512];
  char tmp[20],*t;
  int i;

#ifdef _WIN32
  sprintf(str,"%s",CONFIGFILE);
#else
  if(getenv("HOME") == NULL)
    return 1;
  sprintf(str,"%s/%s",getenv("HOME"),CONFIGFILE);
#endif

  cf = fopen(str,"r");
  if(cf == NULL)
     return 1;

  if(fscanf(cf,"font_size=%d\n",&fsize) != 1)
    return 1;
  if(fscanf(cf,"autoload=%s\n",str) != 1)
    return 1;
  if     (!strcmp(str,"true"))  autoload=true;
  else if(!strcmp(str,"false")) autoload=false; 
  else return 1;
   
  
  fgets(str,512,cf);
  for(i=0;i<(int)strlen("autoload_file=");i++) tmp[i] = str[i];
  tmp[i]='\0';
  if(strcmp(tmp,"autoload_file=") != 0)
   {
    autoload=false; 
    return 1;
   }
  t = str+strlen("autoload_file=");
  i=strlen(t)-1;
  t[i]='\0';
  autoloadfn = t;

  if(fscanf(cf,"niceformat=%s\n",str) != 1)
    return 1;
  if     (!strcmp(str,"true"))  nice=true;
  else if(!strcmp(str,"false")) nice=false;
  else return 1;

  fgets(str,512,cf);
  for(i=0;i<(int)strlen("cdrompath=");i++) tmp[i] = str[i];
  tmp[i]='\0';
  if(strcmp(tmp,"cdrompath=") != 0)
   {
    autoload=false; 
    return 1;
   }
  t = str+strlen("cdrompath=");
  i=strlen(t)-1;
  t[i]='\0';
  cdrompath = t;

#ifndef _WIN32
  if(fscanf(cf,"mounteject=%s\n",str) != 1)
    return 1;
  if     (!strcmp(str,"true"))  mounteject=true;
  else if(!strcmp(str,"false")) mounteject=false; 
  else return 1;
#endif

  fclose(cf);
  return 0;
 }


int CdCatConfig::writeConfig(void)
 {

#ifdef _WIN32
  QFile f (CONFIGFILE);
#else
  if(getenv("HOME") == NULL)
   {
    QMessageBox::warning(0,tr("Error during write config..."),
     tr("I can't get the HOME environment variable.\nIt is necessary couse I'd like to save the config file into your home directory.")
     ,tr("Ok"),0);

    return 1;
   }
   QString filepath=getenv("HOME")+QString("/");
   filepath += CONFIGFILE;
  QFile f(filepath);

#endif

 if (f.open(IO_WriteOnly))
  {
        QTextStream str(&f);    // we will serialize the data into file f
        str.setEncoding(QTextStream::Latin1);
        
	QString fsize_str; 
	
	fsize_str.setNum(fsize);
       
        str << "font_size="+  fsize_str << endl;

         if (autoload) str << "autoload=true"  << endl;
         else          str << "autoload=false" << endl;

         if(autoloadfn.isEmpty())   str << "autoload_file=empty" << endl;
         else                       str << "autoload_file="+autoloadfn << endl;

         if (nice) str << "niceformat=true" << endl;
         else      str << "niceformat=false" << endl;;

         str << "cdrompath="+cdrompath << endl;

#ifndef _WIN32
         if (mounteject) str << "mounteject=true" << endl;
         else      str << "mounteject=false" << endl;;
#endif
         f.close();
        return 0;
  }
 else
  {
#ifdef _WIN32
     QMessageBox::warning(0,tr("Error during write config..."),tr("I can't create or rewrite the ./cdcat.cfg                      ")
     ,tr("Ok"),0);
#else
     QMessageBox::warning(0,tr("Error during write config..."),tr("I can't create or rewrite the $(HOME)/.cdcatconfig                      ")
          ,tr("Ok"),0);
#endif
     return 1;
  }

 }


/************************************************************************************/
ConfigDialog::ConfigDialog( CdCatMainWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    if ( !name )
	setName( "ConfigDialog" );

    p = parent;

    setSizeGripEnabled( TRUE );
    ConfigDialogBaseLayout = new QGridLayout( this, 1, 1, 11, 6, "ConfigDialogBaseLayout");

        layout3 = new QHBoxLayout( 0, 0, 6, "layout3");

    filename = new QLineEdit( this, "filename" );
    layout3->addWidget( filename );

    searchButton = new QPushButton( this, "searchButton" );
    searchButton->setFlat( FALSE );
    layout3->addWidget( searchButton );

    ConfigDialogBaseLayout->addLayout( layout3, 1, 0 );

    line1 = new QFrame( this, "line1" );
    line1->setFrameShape( QFrame::HLine );
    line1->setFrameShadow( QFrame::Sunken );
    line1->setFrameShape( QFrame::HLine );

    ConfigDialogBaseLayout->addWidget( line1, 2, 0 );

    layout4 = new QHBoxLayout( 0, 0, 6, "layout4");

    spinbox = new QSpinBox( this, "spinbox" );
    spinbox->setMaximumWidth(80);
    layout4->addWidget( spinbox );

    lab = new QLabel( this, "lab" );
    layout4->addWidget( lab );

    ConfigDialogBaseLayout->addLayout( layout4, 3, 0 );

    cbNice = new QCheckBox( this, "cbNice" );

    ConfigDialogBaseLayout->addWidget( cbNice, 5, 0 );

    layout5 = new QHBoxLayout( 0, 0, 6, "layout5");

    cdrompath_lineedit = new QLineEdit( this, "cdrompath_lineedit" );
    layout5->addWidget( cdrompath_lineedit );

    cdrom_lab = new QLabel( this, "cdrom_lab" );
    layout5->addWidget( cdrom_lab );

    ConfigDialogBaseLayout->addLayout( layout5, 7, 0 );

    line2 = new QFrame( this, "line2" );
    line2->setFrameShape( QFrame::HLine );
    line2->setFrameShadow( QFrame::Sunken );
    line2->setFrameShape( QFrame::HLine );

    ConfigDialogBaseLayout->addWidget( line2, 4, 0 );

    line3 = new QFrame( this, "line3" );
    line3->setFrameShape( QFrame::HLine );
    line3->setFrameShadow( QFrame::Sunken );
    line3->setFrameShape( QFrame::HLine );

    ConfigDialogBaseLayout->addWidget( line3, 6, 0 );

    line4 = new QFrame( this, "line4" );
    line4->setFrameShape( QFrame::HLine );
    line4->setFrameShadow( QFrame::Sunken );
    line4->setFrameShape( QFrame::HLine );

    ConfigDialogBaseLayout->addWidget( line4, 8, 0 );

    layout6 = new QHBoxLayout( 0, 0, 6, "layout6");
    QSpacerItem* spacer = new QSpacerItem( 110, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    layout6->addItem( spacer );

    okButton = new QPushButton( this, "okButton" );
    okButton->setMinimumSize( QSize( 100, 0 ) );
    okButton->setAutoDefault( TRUE );
    okButton->setDefault( TRUE );
    layout6->addWidget( okButton );

    cancelButton = new QPushButton( this, "cancelButton" );
    cancelButton->setMinimumSize( QSize( 100, 0 ) );
    cancelButton->setAutoDefault( TRUE );
    layout6->addWidget( cancelButton );
    QSpacerItem* spacer_2 = new QSpacerItem( 130, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    layout6->addItem( spacer_2 );

    ConfigDialogBaseLayout->addLayout( layout6, 11, 0 );


    cbAutoload = new QCheckBox( this, "cbAutoload" );

#ifndef _WIN32
    cbMoEj = new QCheckBox( this, "cbMoEj" );
    ConfigDialogBaseLayout->addWidget( cbMoEj, 9, 0 );

    line5 = new QFrame( this, "line5" );
    line5->setFrameShape( QFrame::HLine );
    line5->setFrameShadow( QFrame::Sunken );
    line5->setFrameShape( QFrame::HLine );
    ConfigDialogBaseLayout->addWidget( line5, 10, 0 );

#endif


    ConfigDialogBaseLayout->addWidget( cbAutoload, 0, 0 );
    languageChange();
    resize( QSize(506, 309).expandedTo(minimumSizeHint()) );


    // signals and slots connections
    connect( cancelButton, SIGNAL( clicked() ), this, SLOT( close() ) );
    connect( okButton, SIGNAL( clicked() ), this, SLOT( okExit() ) );
    connect( searchButton, SIGNAL( clicked() ), this, SLOT( ffdbutton() ) );

    cbNice->setChecked(p->cconfig->nice);
    cbAutoload->setChecked(p->cconfig->autoload);
    if(!p->cconfig->autoloadfn.isEmpty())
       filename->setText(p->cconfig->autoloadfn);
    spinbox->setValue(p->cconfig->fsize);
    cdrompath_lineedit->setText(p->cconfig->cdrompath);

#ifndef _WIN32
    cbMoEj ->setChecked(p->cconfig->mounteject);
#endif
}

/*
 *  Destroys the object and frees any allocated resources
 */
ConfigDialog::~ConfigDialog()
{
    // no need to delete child widgets, Qt does it all for us
}


void ConfigDialog::languageChange()
{
    setCaption( tr( "Configure  CdCat..." ) );
    cbAutoload->setText( tr( "Autoload DataBase on startup" ) );
    cbNice->setText( tr( "Save the XML-db.file nicer format(needs more space)" ) );
    cancelButton->setText( tr( "Cancel" ) );
    searchButton->setText("...");
    okButton->setText( tr( "Ok" ) );
    lab->setText(tr("Application font size.(must restart cdcat!) "));
    cdrom_lab->setText( tr("Path to cdrom device") );

#ifndef _WIN32    
    cbMoEj->setText(tr("Scanning: mount cdrom at start / eject when finish"));
#endif
}

void ConfigDialog::ffdbutton()
{
 QString s;
 s = QFileDialog::getOpenFileName(QString::null,tr("CdCat Databases (*.hcf)"),this,
              "autoload file schooser",tr("Choose a file to automatically load!"));
 if(!s.isEmpty())
  filename->setText(s);
}

void ConfigDialog::okExit()
{
  p->cconfig->nice   = cbNice->isChecked();
  p->cconfig->autoload = cbAutoload->isChecked();
  p->cconfig->autoloadfn = filename->text();
  p->cconfig->fsize    = spinbox->value();
  p->cconfig->cdrompath = cdrompath_lineedit->text();

#ifndef _WIN32  
  p->cconfig->mounteject = cbMoEj->isChecked(); 
#endif
  
  p->cconfig->writeConfig();

  close();
}
