/****************************************************************************
                             Hyper's CD Catalog 
		A multiplatform qt and xml based catalog program
  
 Author    : Peter Deak (hyperr@freemail.hu)
 License   : GPL
 Copyright : (C) 2003 Peter Deak
****************************************************************************/


#ifndef CDCAT_DBASE
#define CDCAT_DBASE

#include <qobject.h>

#define HC_UNINITIALIZED 0 
#define HC_CATALOG       1
#define HC_MEDIA         2 
#define HC_DIRECTORY     3 
#define HC_FILE          4 
#define HC_MP3TAG        5 
#define HC_ARCHIV        6 

#define UNKNOWN       0
#define CD            1
#define DVD           2
#define HARDDISC      3 
#define FLOPPY        4
#define NETPLACE      5 
#define FLASHDRV      6
#define OTHERD        7

#define BYTE          0 
#define KBYTE         1
#define MBYTE         2
#define GBYTE         3

#define NAME          1
#define NUMBER        0
#define TYPE          2
#define TIME          3

class QDateTime;
class QFileInfo;
class QString;
class PWw;

class Node 
 {
  public:
  int type; /*
   0 - uninitialized 
   1 - catalog 
   2 - media
   3 - directory     
   4 - file    
   5 - mp3tag
   6 - archive.prop.
  */ 

  Node *next;
  Node *child;
  Node *parent; 
  
  void *data; // see the DB* classes below

  const char *getNameOf(void);  
  QString getFullPath(void);
  void touchDB(void);
  
  Node(void);    
  Node(int t,Node *p);    
  ~Node(void);
 };

class DBCatalog //type is 1
 {
  public:
  char *name;
  char *owner;
  char *comment;
  
  int  writed; //Writed to the disc  1=yes 0=no
  char filename[256];
  char modification[30];
  
  void touch(void);
  
  DBCatalog(void);
  ~DBCatalog(void);
  DBCatalog (char *n,char *o,char *c,char *mod=NULL);
 };
 
class DBMedia  //type is 2
 {
  public:
  char *name;
  int  number;
  char *owner; //if not defined is NULL
  int  type; // 0-unknown 1-CD 2-DVD 3-HardDisc 4-Floppy 5-NetworPlace 6-FlashDrive 7-OtherDevice
  char modification[30];
  char *comment;
  
  DBMedia(char *n,int nu,char *o,int t,char *c,char *mod=NULL);
  DBMedia(void);
   ~DBMedia(void);
 }; 

class DBDirectory //type is 3
 {
  public:
  char *name;
  char modification[30];
  char *comment;
  
  DBDirectory(void);
  DBDirectory(char *n,char *mod,char *c);
  ~DBDirectory(void);
 };
 
class DBFile //type is 4
 {
  public: 
  char *name;
  char modification[30];
  char *comment;
  float size;
  int   sizeType; // 0=byte 1=Kb 2=Mb 3=Gb 
  Node *prop;
  
  DBFile(void);
  DBFile(char *n,char *mod,char *c,float s,int st);
  ~DBFile(void);
 };
 
class DBMp3Tag 
 {
  public:
  char artist [32];
  char title  [32];
  char comment[32];
  char album  [32];
  char year    [5];
  
  DBMp3Tag(void);
  DBMp3Tag(char *a,char *t,char *c,char *al,char *y);
  ~DBMp3Tag(void);
 }; 


class DataBase : public QObject 
 {
  Q_OBJECT  
  private:
   Node *root;
   
  
  public:
  
   QString errormsg;
   bool nicef;
  
   DataBase(void);
   ~DataBase(void);
   void  setDBName (char *n);
   void  setDBOwner(char *o);
   void  setComment(char *c);
   void  setNice   (bool nic);
   
   char *getDBName (void);
   char *getDBOwner(void);
   char *getComment(void);
   Node *getRootNode(void) { return root; }

   int   saveDB       (void);
   int   saveAsDB     (char *filename);
   int   openDB       (char *filename);
   int   insertDB     (char *filename);
   
   int   addMedia     (char *what,char *name,int number,int type,char *owner=NULL);
   void  deleteNode   (Node *d);
   void  sortM        (int mode);
   Node *getMOnPos    (int p);
   
   double        getSize       (Node *s,int level=0);
   unsigned long getCountFiles (Node *s,int level=0);
   unsigned long getCountDirs  (Node *s,int level=0);
   
   PWw *pww; //Pease Wait Window :-)

 
  private:   

   int   scanFsToNode (QString *what,Node *to);
   int   scanFileProp (QFileInfo *fi,DBFile *fc);
   
  
  public:

  /*
   funtions /below/ for make import from another formas:
    The get..  function try to specify a node and return that node's pointer.
    If the retun value is NULL the node is not exist in that environment
    (environment: catalog/needn't to specify of course/ in case getMedia..
                   media in case getDir..  
		   directory in case getFile.. )
    so you have to use the put... function to make the node.
    
    Always test the existion a node before you try to create it !!!
    
    example:
     you want to add "MY_CD/install/sources/cdcat.tar.bz2" where is MY_CD is the media name:
     
      Node *env,*curr;
      curr = getMediaNode("MY_CD");
      if(curr == NULL) curr = putMediaNode("MY_CD", ... );
      
      env = curr;

      curr = getDirectoryNode(env,"install");
      if(curr == NULL) curr = putDirectoryNode(env,"install", ... );
      
      env = curr; //we want to search/insert the "sources" directory in "install" directory.
      
      curr = getDirectoryNode(env,"sources");
      if(curr == NULL) curr = putDirectoryNode(env,"sources", ... );

      env = curr; 
      
      curr = getFileNode(env,"cdcat.tar.bz2");
      if(curr == NULL) curr = putFileNode(env,"cdcat.tar.bz2", ... );
  */
         
   Node * getMediaNode(char *name);
   Node * putMediaNode(char *name,int number,char *owner,int type,char *comment);

   //step only one directory with one call!   
   //the meddir can be media or directoy depend it's will be the first directory or not.
   Node * getDirectoryNode(Node *meddir,char *name);
   Node * putDirectoryNode(Node *meddir,char *name,char *modification,char *comment);
   
   Node * getFileNode(Node *directory,char *name);
   Node * putFileNode(Node *directory,char *name,char *modification,char *comment,int sizeType,float size);
   
   Node * putTagInfo (Node *file,char *artist,char *title,char *comment,char *album,char *year);
   
 };


char *getTime        (QDateTime dt);
char *getCurrentTime (void);
#endif



