/****************************************************************************
                             Hyper's CD Catalog 
		A multiplatform qt and xml based catalog program
  
 Author    : Peter Deak (hyperr@freemail.hu)
 License   : GPL
 Copyright : (C) 2003 Peter Deak
****************************************************************************/

#include "find.h"

#include <qvariant.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qheader.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>

#include <pcre.h>
#include <string.h>
#include <ctype.h>

#include "dbase.h"
#include "mainwidget.h"
#include "wdbfile.h"
#include "guibase.h"
#include "adddialog.h"

void caseSensConversion(char *p);
void easyFormConversion(char *p);

findDialog::findDialog( CdCatMainWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    if ( !name )
	setName( "findDialog" );

    mainw = parent;

    setSizeGripEnabled(TRUE);    
    //Frame
    input=new QFrame(this,"input_frame");
    input->setGeometry( QRect( 0, 0, 505, 160 ) );
    input->setFrameStyle(QFrame::Panel | QFrame::Raised );

    cbOwner = new QComboBox( FALSE, input, "cbOwner" );
    cbOwner->setGeometry( QRect( 80, 90, 160, 26 ) );

    cbSin = new QComboBox( FALSE, input, "cbSin" );
    cbSin->setGeometry( QRect( 80, 60, 160, 26 ) );

    buttonCancel = new QPushButton( input, "buttonCancel" );
    buttonCancel->setGeometry( QRect( 260, 130, 98, 25 ) );

    buttonOk = new QPushButton( input, "buttonOk" );
    buttonOk->setGeometry( QRect( 130, 130, 98, 25 ) );

    textLabel3 = new QLabel( input, "textLabel3" );
    textLabel3->setGeometry( QRect( 10, 10, 52, 20 ) );

    textLabel1 = new QLabel( input, "textLabel1" );
    textLabel1->setGeometry( QRect( 10, 60, 70, 20 ) );

    textLabel2 = new QLabel( input, "textLabel2" );
    textLabel2->setGeometry( QRect( 10, 90, 70, 20 ) );

    leText = new QLineEdit( input, "leText" );
    leText->setGeometry( QRect( 60, 10, 420, 20 ) );

    cbComment = new QCheckBox( input, "cbComment" );
    cbComment->setGeometry( QRect( 280, 100, 77, 20 ) );
    cbComment->setChecked( TRUE );

    cbFilename = new QCheckBox( input, "cbFilename" );
    cbFilename->setGeometry( QRect( 280, 80, 77, 20 ) );
    cbFilename->setChecked( TRUE );

    cbDirname = new QCheckBox( input, "cbDirname" );
    cbDirname->setGeometry( QRect( 280, 60, 100, 20 ) );
    cbDirname->setChecked( TRUE );

    cbTcomm = new QCheckBox( input, "cbTcomm" );
    cbTcomm->setGeometry( QRect( 380, 120, 120, 20 ) );
    cbTcomm->setChecked( TRUE );

    cbAlbum = new QCheckBox( input, "cbAlbum" );
    cbAlbum->setGeometry( QRect( 380, 100, 100, 20 ) );
    cbAlbum->setChecked( TRUE );

    cbTitle = new QCheckBox( input, "cbTitle" );
    cbTitle->setGeometry( QRect( 380, 80, 90, 20 ) );
    cbTitle->setChecked( TRUE );

    cbArtist = new QCheckBox( input, "cbArtist" );
    cbArtist->setGeometry( QRect( 380, 60, 100, 20 ) );
    cbArtist->setChecked( TRUE );

    cbCasesens = new QCheckBox( input, "cbCasesens" );
    cbCasesens->setGeometry( QRect( 60, 30, 180, 20 ) );
    cbCasesens->setChecked( FALSE );

    cbEasy = new QCheckBox( input, "cbEasy" );
    cbEasy->setGeometry( QRect( 250, 30, 200, 20 ) );
    cbEasy->setChecked( TRUE );

    input->setMaximumSize(505, 160);
    input->setMinimumSize(505, 160);
    //end Frame

    textLabel5 = new QLabel( this, "textLabel5" );
    textLabel5->setMaximumWidth(55);

    line1 = new QFrame( this, "line1" );
    line1->setFrameShape( QFrame::HLine );
    line1->setFrameShadow( QFrame::Sunken );
    line1->setFrameShape( QFrame::HLine );

    line2 = new QFrame( this, "line2" );
    line2->setFrameShape( QFrame::HLine );
    line2->setFrameShadow( QFrame::Sunken );
    line2->setFrameShape( QFrame::HLine );

    resultsl = new QListView( this, "resultsl" );
    resultsl->addColumn( tr( "Name" ));
    resultsl->addColumn( tr( "Type" ));
    resultsl->addColumn( tr( "Media"));
    resultsl->addColumn( tr( "Path" ));
    resultsl->addColumn( tr( "Modification" ) );
    resultsl->setAllColumnsShowFocus(true);

    buttonClose = new QPushButton( this, "buttonClose" );
    
    v1 = new QVBoxLayout(this);
    h1 = new QHBoxLayout();
    h2 = new QHBoxLayout();
    h3 = new QHBoxLayout();
    h4 = new QHBoxLayout();
    

    h1->insertStretch(0);
    h1->insertWidget (1,input);
    h1->insertStretch(2);
    
    h2->insertSpacing(0,5);
    h2->insertWidget (1,line1);
    h2->insertSpacing(2,5);
    h2->insertWidget (3,textLabel5);
    h2->insertSpacing(4,5);
    h2->insertWidget (5,line2);
    h2->insertSpacing(6,5);
    
    h3->insertSpacing(0,5);
    h3->insertStretch(1);
    h3->insertWidget (2,buttonClose);
    h3->insertStretch(3);
    h3->insertSpacing(4,5);
    
    h4->insertSpacing(0,5);
    h4->insertWidget (1,resultsl);
    h4->insertSpacing(2,5);
    
    v1->insertSpacing(0,2);
    v1->insertLayout (1,h1);
    v1->insertSpacing(2,2);
    v1->insertLayout (3,h2);
    v1->insertSpacing(4,2);
    v1->insertLayout (5,h4);
    v1->insertSpacing(6,3);
    v1->insertLayout (7,h3);
    v1->insertSpacing(8,5);

    languageChange();
    resize( QSize(515, 365).expandedTo(minimumSizeHint()) );

    
    fillCBox();
    connect(buttonCancel,SIGNAL(clicked()),this,SLOT(cancele()));
    connect(buttonOk,SIGNAL(clicked()),this,SLOT(seeke()));
    connect(buttonClose,SIGNAL(clicked()),this,SLOT(closee()));
    connect(resultsl,SIGNAL(currentChanged(QListViewItem *)),this,SLOT(select(QListViewItem *)));
    setTabOrder(leText,buttonOk);
    leText->setFocus();
}

/*
 *  Destroys the object and frees any allocated resources
 */
findDialog::~findDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

void findDialog::languageChange()
{
    setCaption( tr( "Search in the database..." ) );
    buttonClose->setText( tr( "Close / Go to selected" ) );
    buttonCancel->setText( tr( "Cancel" ) );
    buttonOk->setText( tr( "OK" ) );
    textLabel3->setText( tr( "Find:" ) );
    textLabel1->setText( tr( "Seek in:" ) );
    textLabel2->setText( tr( "Owner:" ) );
    textLabel5->setText( tr( "Resoults" ) );

    resultsl->clear();

    cbComment->setText( tr( "Comment" ) );
    cbFilename->setText( tr( "File name" ) );
    cbDirname->setText( tr( "Directory name" ) );
    cbTcomm->setText( tr( "mp3-tag Comment" ) );
    cbAlbum->setText( tr( "mp3-tag Album" ) );
    cbTitle->setText( tr( "mp3-tag Title" ) );
    cbArtist->setText( tr( "mp3-tag Artist" ) );
    cbCasesens->setText( tr( "Case sensitive" ) );
    cbEasy->setText( tr( "Use easy matching instead regex" ) );
}
/***************************************************************************/
int findDialog::fillCBox(void)
 {
  char ow[256];
  int  i,f,c;
  Node *tmp=mainw->db->getRootNode();
  
  if(tmp == NULL) return 0;
   
  cbOwner->clear();
  cbOwner->insertItem(tr("All/Everybody"));
  cbOwner->insertItem(((DBCatalog *)(tmp->data))->owner);
  
  cbSin  ->clear();
  cbSin  ->insertItem(tr("All media"));
  
  tmp=tmp->child; //Jump to the first media
  while(tmp != NULL)
   {
    cbSin  ->insertItem(tmp->getNameOf());
    strcpy(ow,((DBMedia *)(tmp->data))->owner);
    c = cbOwner->count();
    for(i=0,f=1;c>i;i++)
      if(strcmp(ow,(const char *)cbOwner->text(i)) == 0) 
        f=0;
    if(f)
     cbOwner->insertItem(ow);    
    tmp=tmp->next;
   }
  return 0;  
 }
 
int findDialog::select(QListViewItem *i)
 {
  if(i == NULL)
     return 0;

  char loc[1024];
  
  strcpy(loc,i->text(3));
  mainw->guis->updateListFromNode(
         (mainw->guis->getNodeFromFullName(mainw->db->getRootNode(),QString(loc+2)))); 
  for(QListViewItemIterator it=mainw->listView->firstChild();it.current();it++)
   {
     if(strcmp((it.current())->text(0),i->text(0)) ==0)
        mainw->listView->setCurrentItem(it.current());       
   }
  mainw->listView->curr_vis();  
  mainw->listView->repaint();  
  return 0;
 }
  
/***************************************************************************/
int findDialog::closee(void)
 {
  close ();
  return 0;
 }
/***************************************************************************/
int findDialog::cancele(void)
 {
  close();
  return 0;
 }
int findDialog::seeke(void)
 {
  seekEngine *se;

  if(mainw == NULL && mainw->db == NULL) 
    return 0;
  
  if((leText->text()).isEmpty()) 
    return 0;

  se = new seekEngine(this);
  se->start_seek();
  delete se;
  return 0;
 }
/***************************************************************************/ 
void caseSensConversion(char *p)
 {
  int s,t,ss = strlen(p);
  char *tmp=new char[2048];
  
  for(s=0,t=0;s<=ss;s++,t++)
   {
    if((p[s] >= 'a' && p[s]<='z') || 
       (p[s] >= 'A' && p[s]<='Z'))
         {
	  tmp[t++]='[';
	  tmp[t++]=tolower(p[s]);
	  tmp[t++]='|';
	  tmp[t++]=toupper(p[s]);	  
	  tmp[t  ]=']';
	 }
    else
         {
	  tmp[t]=p[s];
	 }	 
   } 
  strcpy(p,tmp);    
  delete [] tmp;
 }

/***************************************************************************/
void easyFormConversion(char *p)
 {
  int s,t,ss = strlen(p);
  char *tmp=new char[2048];
  
  for(s=0,t=0;s<=ss;s++,t++)
   {
    if     (p[s] == '?')
         {
	   tmp[t  ]='.';
	 }
    else if(p[s] == '*')
         {
	  tmp[t++]='.';
	  tmp[t  ]='*';
	 }
    else if((p[s] >= 'a' && p[s]<='z') || 
            (p[s] >= 'A' && p[s]<='Z') ||
	    (p[s] >= '0' && p[s]<='9'))
         {
	  tmp[t]=p[s];
	 }
    else if(p[s] == '\0')	 
         {
	  tmp[t]='\0';
	 }
    else 
         {
	  tmp[t++]='\\';  	 
	  tmp[t  ]=p[s];  
	 }	 	 
   } 
  sprintf(p,"^%s$",tmp);    
  delete [] tmp;   
}

/***************************************************************************

  Seek engine Class

***************************************************************************/

seekEngine::seekEngine(findDialog *fdp)
 {
  fd   = fdp;
  patt = new char[2048];
 }
/***************************************************************************/
seekEngine::~seekEngine(void)
 {
  delete [] patt;
 }
/***************************************************************************/ 
int seekEngine::start_seek(void)
 {
 /*get the pattern*/
  strcpy(patt,(const char *)fd->leText->text());
     
  /*recode the pattern /easy/  put ^$ and ? -> .  * -> .* */
  if(fd->cbEasy->isChecked())
     easyFormConversion(patt);

  /*recode the pattern /Case sens/ lok -> [l|L][o|O][k|K] */
  if(!fd->cbCasesens->isChecked())
     caseSensConversion(patt);

 //fprintf(stderr,"The complete pattern is \"%s\"\n",patt);

  re = pcre_compile(patt,0,&error,&errptr,NULL);

  if(re == NULL)
   {
    sprintf(patt,"%s: %d",error,errptr);
    QMessageBox::warning(fd,tr("Error in the pattern:"),patt,tr("Ok"),0);
    return 1;
   }

  hints = pcre_study(re,0,&error);

  if(error != NULL)
   {
    QMessageBox::warning(fd,tr("Error in the pattern:"),error,tr("Ok"),0);
    return 1;
   }
  
  pww=new PWw(fd);
  progress(pww);
  
  fd->resultsl->clear();
  founded = 0;
   
  dirname  = fd->cbDirname ->isChecked();
  filename = fd->cbFilename->isChecked();
  comment  = fd->cbComment ->isChecked();
  tartist  = fd->cbArtist  ->isChecked();
  ttitle   = fd->cbTitle   ->isChecked();
  tcomment = fd->cbTcomm   ->isChecked();
  talbum   = fd->cbAlbum   ->isChecked();
  
  allmedia = false;
  allowner = false;

  strcpy(media,(const char *)(fd->cbSin  ->currentText()));
  strcpy(owner,(const char *)(fd->cbOwner->currentText()));

  if(strcmp((const char *)tr("All/Everybody"),owner) == 0)
   allowner = true;
  if(strcmp((const char *)tr("All media")    ,media) == 0)
   allmedia = true;

  progress(pww);    
  /*seek...*/
  analyzeNode(fd->mainw->db->getRootNode());

  if(founded == 0)
  fd->resultsl->insertItem(new QListViewItem(fd->resultsl,tr("There is no matching.")));


  progress(pww);  
  pww->end();
  delete pww;
  return 0;
 }
/***************************************************************************/ 
int seekEngine::analyzeNode(Node *n,Node *pa)
 {
  
   if(n == NULL) return 0; 
   switch(n->type)
    {
       case HC_CATALOG:
           analyzeNode(n->child);
           return 0;
       
       case HC_MEDIA:
           progress(pww);
          
	   if((allmedia || (strcmp(media,n->getNameOf())==0)) &&
              (allowner || (strcmp(owner,((DBMedia *)(n->data))->owner)==0))) 
                  analyzeNode(n->child);	    
           analyzeNode(n->next );	    	   
           return 0;
	   
       case HC_DIRECTORY:
           progress(pww);
         
	   if(dirname)  
	     if(matchIt(n->getNameOf()))
               {
	        putNodeToList(n);
		analyzeNode(n->child);    
  	        analyzeNode(n->next);    
                return 0;
	       }
	       
           if(comment)
	     if(matchIt(((DBDirectory *)(n->data))->comment))
	        putNodeToList(n);
	       
	   analyzeNode(n->child);    
	   analyzeNode(n->next);    
           return 0;
	   
       case HC_FILE:
           if(filename) 
	     if(matchIt(n->getNameOf()))
	      {
               putNodeToList(n);
  	       analyzeNode(n->child);    
	       analyzeNode(n->next);    
               return 0;
	      }
	   if(comment)
	     if(matchIt(((DBFile *)(n->data))->comment))
	      {
               putNodeToList(n);
  	       analyzeNode(n->child);    
	       analyzeNode(n->next);    
               return 0;

	      }	
       
	   analyzeNode(((DBFile *)(n->data))->prop,n);
	   analyzeNode(n->child);    
	   analyzeNode(n->next);    
           return 0;
	   
       case HC_MP3TAG:
           if(tartist)
	     if(matchIt(((DBMp3Tag *)(n->data))->artist))
	       {
	        putNodeToList(pa);
		return 0;
	       }
           if(ttitle)
	     if(matchIt(((DBMp3Tag *)(n->data))->title))
	       {
	        putNodeToList(pa);
		return 0;
	       }
           if(talbum)
	     if(matchIt(((DBMp3Tag *)(n->data))->album))
	       {
	        putNodeToList(pa);
		return 0;
	       }
           if(tcomment)
	     if(matchIt(((DBMp3Tag *)(n->data))->comment))
	       {
	        putNodeToList(pa);
		return 0;
	       }
           return 0;
    }
  return -1;
 } 

int  seekEngine::matchIt(const char *txt)
 {
  if(txt == NULL) return 0;

  int match = pcre_exec(re,hints,txt,strlen(txt),0,0,offsets,99);
  if(match == 1) return 1;
  return 0;
 }

void seekEngine::putNodeToList(Node *n)
 {
  
  Node *tmp;
  char type[30];
  char media[128];
  char mod[30];
  
  if(n == NULL) return;
  
  switch(n->type)
   {
    case HC_DIRECTORY: strcpy(type,(const char *)tr("dir"));
                       strcpy(mod,((DBDirectory *)(n->data))->modification);    
        break;
    case HC_FILE:      sprintf(type,"%s/ %.1f%s",(const char *)tr("file"),
                                ((DBFile *)(n->data))->size,
                                getSType(((DBFile *)(n->data))->sizeType));
                       strcpy(mod,((DBFile *)(n->data))->modification);    				
	break;
    default:           strcpy(type,(const char *)tr("error")); 
        break;
   }
  tmp=n;
  while(tmp->type != HC_MEDIA)
   tmp=tmp->parent;
  
  sprintf(media,"%s/%d",tmp->getNameOf(),((DBMedia *)(tmp->data))->number);      
  
  fd->resultsl->insertItem(new QListViewItem(fd->resultsl,
                                            n->getNameOf(),type,media,
					    n->getFullPath(),mod
                                           ));
  progress(pww);					   
  founded++;
 }

/***************************************************************************/ 

