/****************************************************************************
                             Hyper's CD Catalog
		A multiplatform qt and xml based catalog program
  
 Author    : Peter Deak (hyperr@freemail.hu)
 License   : GPL
 Copyright : (C) 2003 Peter Deak
****************************************************************************/

#include <string.h>

#include <qlistview.h>
#include <qimage.h>
#include <qlabel.h>
#include <qheader.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qapplication.h>

#ifndef _WIN32
#ifndef _OLD_QT_
#include <qprocess.h>
#endif
#endif

#include "cdcat.h"
#include "guibase.h"
#include "mainwidget.h"
#include "dbase.h"
#include "cdcat.h"
#include "recode.h"
#include "wdbfile.h"
#include "commwidget.h"
#include "hdirview.h"
#include "newdbdialog.h"
#include "adddialog.h"
#include "find.h"
#include "config.h"
#include "help.h"
#include "import.h"

#include "icons.h"

char *rbuff=NULL;

GuiSlave::GuiSlave(CdCatMainWidget *p)
 {
  mainw=p;
  
  standON = NULL;
  tmpParent = NULL;
  if(mainw->db != NULL)
     updateListFromNode(mainw->db->getRootNode());

 }

void GuiSlave::updateStatusl(Node *n)
 {
  QString o("");
  if(n == NULL)
   {
     if(mainw->db == NULL)
        mainw->statusl->setText(tr("No opened database."));
     else
        mainw->statusl->setText(tr("No selected item."));
     return;
   }     
  mainw->statusl->setText(n->getFullPath());
 }

int GuiSlave::isIdentical(int i)
 {
  Node *t;
   
  if(mainw->db == NULL) return 1;
  t=mainw->db->getRootNode();
  if(t->child == NULL) return 1;
  t = t->child;
  while(t != NULL)
   {
      if(((DBMedia *)(t->data))->number == i) return 0;
   
      t = t->next;
   }
  return 1;
 }

int GuiSlave::isIdentical(QString q)
 {
  Node *t;
   
  if(mainw->db == NULL) return 1;
  t=mainw->db->getRootNode();
  if(t->child == NULL) return 1;
  t = t->child;
  while(t != NULL)
   {
      if(strcmp(((DBMedia *)(t->data))->name,(const char *)q) == 0) return 0;
   
      t = t->next;
   }
  return 1;
 }

Node *GuiSlave::getNodeFromFullName(Node *root,const QString& newloc)
 {
  int i=0,ki=0;
  char *org;
  char *re_name;
  char name[256];
  /*------------*/
  Node *tmp=NULL;
  Node *step =  root;
  int   down = 0;
  org=re_name=mstr((const char *)newloc);
  
  while(!ki)
   {
    for(i=0;re_name[i]!='/' && re_name[i]!='\0';i++ )
        name[i]=re_name[i];
    if(re_name[i]=='\0')
     {
       ki=1;   
       continue;
     }
    name[i]='\0';       // <- ittvan 
    
    if(down == 0 )
     {
      if(strcmp(name,step->getNameOf()))
         return NULL; /*error Not matching catalog name!*/
      down++; 
     }
    else
     {
      tmp=step->child;
      if(tmp==NULL) 
	  return NULL;  //I couldn't find the requested element!
      
      while(strcmp(tmp->getNameOf(),name)) 
       {
        tmp=tmp->next;
        if(tmp==NULL) 
  	    return NULL;  //I couldn't find the requested element!
	down++; 
       }
      step=tmp; 
     } 
    re_name+=(strlen(name)+1);
   }
  delete [] org;
  return step; 
 }

int GuiSlave::listUpdate(const QString& newloc)
 {
   if(mainw->db == NULL) return 0;
   Node *pdir = getNodeFromFullName(mainw->db->getRootNode(),newloc);
   updateStatusl(pdir);
   mainw->commentWidget->showNode(pdir,0);
   standON = NULL;
   return updateListFromNode(pdir);
 }

int GuiSlave::updateListFromNode(Node *pdir)
 {
   
   Node *tmp;
   QListViewItem *lvi=NULL,*after=NULL;
   char str1[128];
   char str2[36];
   
   if(pdir == NULL)
     pdir = NodePwd;
   
   if(pdir == NULL) 
    {
     mainw->listView->clear();
     return 0;
    }

   mainw->listView->clear();
   mainw->listView->setSorting(-1);
   if(pdir->parent != NULL)
    {
     lvi = new QListViewItem(mainw->listView,"..","",tr("Directory"));
     lvi->setPixmap(0,*get_v_back_icon());
     lvi->setExpandable(FALSE);
     after = lvi;
    }
   
   NodePwd = pdir;
   tmp=pdir->child;

 /*List everithing but files:*/   

   while(tmp != NULL)
    {
     if(tmp->type == HC_FILE)
      {
        tmp=tmp->next;
	continue;
      }
      if(tmp->type == HC_MP3TAG)
        return 1; //Error
       
      // 2.(size) Column name:
        sprintf(str1," ");	

      // 3. Column name:
      switch(tmp->type)
        {
         case HC_CATALOG : strcpy(str2,tr("Catalog")); break;
	 case HC_MEDIA   :
	            switch(((DBMedia*)(tmp->data))->type)
	 	     {
	               case UNKNOWN : strcpy(str2,tr("Unknown(DB)"));         break;
                       case CD      : strcpy(str2,tr("CD"));   break;
                       case DVD     : strcpy(str2,tr("DVD"));  break;
                       case HARDDISC: strcpy(str2,tr("HardDisc"));   break;
                       case FLOPPY  : strcpy(str2,tr("Floppy"));   break;
                       case NETPLACE: strcpy(str2,tr("NetworkDrv"));   break;
                       case FLASHDRV: strcpy(str2,tr("FlashDrv"));   break;
                       case OTHERD  : strcpy(str2,tr("OtherDevice"));   break;
		     }
		break;
         case HC_DIRECTORY: strcpy(str2,tr("Directory")); break;
       }



      lvi = new QListViewItem(mainw->listView,after,tmp->getNameOf(),str1,str2);
      after = lvi;
      switch(tmp->type)
       {
         case HC_CATALOG : break;
         case HC_MEDIA:
                   switch(((DBMedia*)(tmp->data))->type)
		    {
		      case UNKNOWN : lvi->setPixmap(0,*get_m_unknown_icon()); break;        
                      case CD      : lvi->setPixmap(0,*get_m_cd_icon()); break; 
                      case DVD     : lvi->setPixmap(0,*get_m_dvd_icon()); break; 
                      case HARDDISC: lvi->setPixmap(0,*get_m_hdd_icon()); break; 
                      case FLOPPY  : lvi->setPixmap(0,*get_m_floppy_icon()); break; 
                      case NETPLACE: lvi->setPixmap(0,*get_m_net_icon()); break; 
                      case FLASHDRV: lvi->setPixmap(0,*get_m_flash_icon()); break; 
                      case OTHERD  : lvi->setPixmap(0,*get_m_other_icon()); break; 
		    }
         break;
       case HC_DIRECTORY: lvi->setPixmap( 0,*get_v_folderclosed_icon());  break;
     }
      
     lvi->setExpandable(FALSE);
     if(tmpParent != NULL)/*Return to previous parent*/
      {
       if(!strcmp(tmp->getNameOf(),tmpParent->getNameOf()))
        { 
	 mainw->listView->setCurrentItem(lvi);
         tmpParent = NULL;
	}
      }    
     tmp=tmp->next;
    }

 /*List files:*/
   tmp=pdir->child;    
   while(tmp != NULL)
    {
      if(tmp->type != HC_FILE)
       {
        tmp=tmp->next;
        continue;	
       }
       
      /*2.(size) Column name:*/       
      sprintf(str1,"%.2f %s",((DBFile *)(tmp->data))->size,
	          getSType(((DBFile *)(tmp->data))->sizeType));

      strcpy(str2,tr("File"));

      lvi = new QListViewItem(mainw->listView,after,tmp->getNameOf(),str1,str2);
      after = lvi;
      lvi->setPixmap( 0,*get_v_file_icon() );  
      lvi->setExpandable(FALSE);

      tmpParent =NULL;
      tmp=tmp->next;
    }
   return 0; 
 }

int GuiSlave::standOn(QListViewItem *on)
 {
  Node *tmp;

  if(on == NULL || on == 0) return 0;
  if(!strcmp(on->text(0),".."))
   {
    mainw->commentWidget->showNode(NodePwd->parent,1);
    updateStatusl(NodePwd->parent);
    return 0;
   }    

  tmp=NodePwd->child;
  while(strcmp(tmp->getNameOf(),on->text(0)))
    {
     tmp=tmp->next;
     if(tmp == NULL)
             return 0;	 
    }

  standON = tmp;
  mainw->commentWidget->showNode(tmp,0);
  updateStatusl(tmp);
  return 0;
 }

int GuiSlave::doubleClickOn(QListViewItem *on)
 {
   Node *tmp;
   if(!strcmp(on->text(0),".."))
     {
      if(NodePwd->parent != NULL)
       {
         tmpParent =  NodePwd;
         NodePwd   =  NodePwd->parent; 
       }
     }
    else
     { //step down
       tmp=NodePwd->child;
       while(strcmp(tmp->getNameOf(),on->text(0)))
        {
	 tmp=tmp->next;
	 if(tmp == NULL) return 0;	 
	}
       if(tmp->type == HC_FILE)	
          return 0;    
       NodePwd = tmp;
     } 
    updateListFromNode();     
    return 0;
 }

void GuiSlave::panelsOFF(void)
 {
  mainw->DirView->clear();
  mainw->listView->clear();
  if(mainw->db != NULL)
     tmpParent=NodePwd=mainw->db->getRootNode();
  else      
     tmpParent=NodePwd=NULL;
  mainw->commentWidget->act = NULL;
 }

int  GuiSlave::cHcaption(void)
 {
  if((mainw->db != NULL) && (((DBCatalog *)(mainw->db->getRootNode()->data))->writed == 0 ))
     mainw->setCaption( tr ("Hyper's CD Catalogizer (modified)"));
   else 
     mainw->setCaption( tr ("Hyper's CD Catalogizer"));
  return 0;     
 }

void GuiSlave::panelsON(void)
 {
  panelsOFF(); //That case you forget the OFF before...
  
  mainw->DirView->start();
  mainw->listView->start();
  updateListFromNode();
  mainw->commentWidget->act = NULL;
  mainw->commentWidget->repaint();
  updateStatusl(standON); 
  cHcaption(); 
 }

/*****************************************************************************
 *
 *  class HQListView
 * 
*****************************************************************************/
void HQListView::start(void)
 {
  ;
 }

HQListView::HQListView(QWidget *parent,const char *name,WFlags f)
:QListView(parent,name,f)
 {
  mainw = (CdCatMainWidget *) parent;
  
  setSelectionMode(Single);
  addColumn(tr("Name"));
  addColumn(tr("Size"));  
  addColumn(tr("Type"));
  setAllColumnsShowFocus(true);
 }

void HQListView::curr_vis(void)
 {
  QListViewItem * i;
  if((i=currentItem()) != 0)
    ensureItemVisible(i);
 }


int GuiSlave::hotKeys(QKeyEvent *ke)
 {
  switch(ke->key())
   {
    case (Qt::Key_O) :  openEvent();    break;
    case (Qt::Key_N) :  newEvent();     break;    
    case (Qt::Key_S) :  saveEvent();    break;        
    case (Qt::Key_C) :  closeEvent();   break;        
    case (Qt::Key_A) :  addEvent();     break;    
    case (Qt::Key_R) :  rescanEvent();  break;        
    case (Qt::Key_G) :  configEvent();  break;
    case (Qt::Key_F) :  findEvent();    break;        
    case (Qt::Key_H) :  helpEvent();    break;        
    case (Qt::Key_Q) :  mainw->close(); break;
    case (Qt::Key_Space) :  sizeEvent();    break;
   }
  return 0; 
 }

void HQListView::keyPressEvent(QKeyEvent *ke)
 {
  Node *tmp;
  QListViewItem *it;


  mainw->guis->hotKeys(ke); 
  
  if(mainw->db == NULL) return;
  
  if(ke->key() == Qt::Key_Left || ke->key() == Qt::Key_Backspace) 
   {
    if(mainw->guis->NodePwd->parent != NULL)
     {
      mainw->guis->tmpParent =  mainw->guis->NodePwd; 
      mainw->guis->NodePwd   = mainw->guis->NodePwd->parent; 
     }

    mainw->guis->updateListFromNode();       
   }

  if(ke->key() == Qt::Key_Right || ke->key() == Qt::Key_Return )
   {
    it = currentItem();
    if(!strcmp(it->text(0),".."))
     {

      if(mainw->guis->NodePwd->parent != NULL)
       {
         mainw->guis->tmpParent =  mainw->guis->NodePwd;
         mainw->guis->NodePwd   = mainw->guis->NodePwd->parent; 
       }
     }
    else
     { //step down
       tmp=mainw->guis->NodePwd->child;
       while(strcmp(tmp->getNameOf(),it->text(0)))
        {
	 tmp=tmp->next;
	 if(tmp == NULL) return;	 
	}
       if(tmp->type == HC_FILE)	
          return;    
       mainw->guis->NodePwd = tmp;
     }

    mainw->guis->updateListFromNode();     
   }
   
  curr_vis();        
  if(ke->key() == Qt::Key_Left || ke->key() == Qt::Key_Right)
   return;

  QListView::keyPressEvent(ke);
 }

/* Mainwindow buttons ***********************************/
int GuiSlave::newEvent(void)
 {
  newdbdialog *d = new newdbdialog(mainw,"new_db_dialog",true);

  closeEvent();
  d->exec();
  if(d->OK == 1)
   {
    panelsOFF();
    if(mainw->db == NULL)
        mainw->db = new DataBase();
    mainw->db->setDBName ((char *)((const char *)d->nameD ));
    mainw->db->setDBOwner((char *)((const char *)d->ownerD));
    mainw->db->setComment(recode((char *)((const char *)d->commD ),&rbuff));
    panelsON();
   }    
  return 0;
 }


int GuiSlave::openEvent(void)
 {
  char fnc[256];
  QString fn;

  fn = QFileDialog::getOpenFileName(NULL,tr("CdCat databases (*.hcf )"),mainw,"openfiled",tr("Open a file..."));
  if(fn.isEmpty())  return 0;
  
  strcpy(fnc,(const char *)fn);
  closeEvent();
  panelsOFF();
  

  if(mainw->db == NULL)
     mainw->db = new DataBase();
  
  PWw *pww = new PWw(mainw);
  mainw->db->pww = pww;     
  progress(pww);

  if(mainw->db->openDB(fnc) != 0 )   
   { // An error occured
     QMessageBox::warning(mainw,tr("Error while opening..."),
                          mainw->db->errormsg,tr("Ok"),0); 
     mainw->db = NULL;			  
     standON = NodePwd = NULL;;			  
     panelsOFF();
   }
  progress(pww);
  
  panelsON();
  
  progress(pww);
  pww->end();
  if(mainw->db != NULL)
     mainw->db->pww = NULL;
  delete pww;
  return 0;
 }
 
int GuiSlave::saveEvent(void)
 {
  if(mainw->db == NULL) return 0;

  PWw *pww = new PWw(mainw);
  mainw->db->pww = pww;     
  progress(pww);

  mainw->db->setNice(mainw->cconfig->nice);
  if(mainw->db->saveDB() != 0 )   
   {
    progress(pww);
    pww->end();
    mainw->db->pww = NULL;
    delete pww;
    
    saveasEvent();
    return 0;
   }
        
  panelsOFF();
  panelsON();
  
  progress(pww);
  pww->end();
  mainw->db->pww = NULL;
  delete pww;
  
  return 0;  
 }

int GuiSlave::saveasEvent(void)
 {
  char fnc[256];
  QString fn;
 
  if(mainw->db == NULL) return 0;
  fn = QFileDialog::getSaveFileName(NULL,tr("CdCat databases (*.hcf )"),mainw,"savefdlg",tr("Save to file..."));
  if(fn.isEmpty())  return 0;


  PWw *pww = new PWw(mainw);
  mainw->db->pww = pww;     
  progress(pww);

  
  strcpy(fnc,(const char *)fn);
  
  //extension correction if necessary
  if(strlen(fnc)<5 || strcmp((fnc+(strlen(fnc)-4)),".hcf") != 0)
    sprintf(fnc,"%s.hcf",(const char *)fn);

  progress(pww);

  mainw->db->setNice(mainw->cconfig->nice);
  if(mainw->db->saveAsDB(fnc) != 0 )   
   { // An error occured
     QMessageBox::warning(mainw,tr("Error while saving..."),mainw->db->errormsg,tr("Ok"),0); 
   }
  panelsOFF();
  progress(pww);
  panelsON();     

  progress(pww);
  pww->end();
  mainw->db->pww = NULL;
  delete pww;
  return 0;
 }
 
int GuiSlave::closeEvent(void)
 {
  if(mainw->db == NULL) return 0;
  if(((DBCatalog *)(mainw->db->getRootNode()->data))->writed == 0 )
    if(QMessageBox::warning(mainw,tr("Closing previous database..."),tr("Do you want to save the changes?"),
        tr("Yes"),tr("No")) == 0)
        if(mainw->db->saveDB() != 0)
	  saveasEvent();
	
  panelsOFF();	  

  /* Freeing database object */
  delete mainw->db;
  mainw->db = NULL;
  standON = NodePwd = NULL;
  panelsON();
  return 0;
 }

int GuiSlave::deleteEvent(void)
 {
  QString o;
  if(mainw->db == NULL) return 0;
  if(standON == NULL)
   {
     QMessageBox::warning(mainw,tr("Error:"),tr("There is no selected item on the middle list box!"),
                          tr("Ok"),0);
     return 0;
   }

  o = tr("Do you really want to delete this?: \n%1").arg(standON->getNameOf());
  if(QMessageBox::warning(mainw,tr("Confirmation"),o,
        tr("Yes"),tr("No")) == 0)
   {
    panelsOFF();
    mainw->db->deleteNode(standON);
    standON = NULL;
    panelsON();
   }
  return 0;
 }

int GuiSlave::addEvent(void)
 {
   int i;
   if(mainw->db == NULL)
      newEvent();
   if(mainw->db == NULL)
      return 0;


   addDialog *d=new addDialog(this,mainw,"adddialog",true);
   d->exec();

#ifndef _WIN32
#ifndef _OLD_QT_
    if(mainw->cconfig->mounteject)
    if (d->type == CD || d->type == DVD) 
    {
        QApplication::setOverrideCursor(waitCursor);
        process = new QProcess();
        process->addArgument("mount");
        process->addArgument(d->dDir);
        if (!process->start())
                QMessageBox::warning(0,tr("Cannot mount CD"),tr("Cannot mount CD"));
        else {
            while(process->isRunning())
                mainw->app->processEvents();
            process->clearArguments();
            delete (process);
            QApplication::restoreOverrideCursor();
        }
    }
#endif
#endif

   PWw *pww = new PWw(mainw);
   mainw->db->pww = pww;
   progress(pww);

   if(d->OK == 1)
    {
     panelsOFF();
     i=  mainw->db->addMedia((char *)((const char *)d->dDir),
                            (char *)((const char *)d->dName),
			    d->serial,d->type,
			    (d->dOwner).isEmpty() ? NULL : ((char *)((const char *)(d->dOwner))));
     panelsON();
     if(i!=0)
      {
        QMessageBox::warning(mainw,
	tr("Warning..."),tr("There was some error during the scanning,\nthe DataBase may be incomplete"),
                             tr("Ok"),0);
      }
     if(!(d->dComm).isEmpty())
      {
       Node *tn=(mainw->db->getRootNode())->child;
       while(strcmp(tn->getNameOf(),(const char *)d->dName) != 0) tn=tn->next;
       ((DBMedia *)(tn->data))->comment = mstr(recode((char *)((const char *)d->dComm),&rbuff));
      }
    }
  progress(pww);
  pww->end();
  mainw->db->pww = NULL;
  delete pww;

#ifndef _WIN32
#ifndef _OLD_QT_
  if(mainw->cconfig->mounteject)
  if (d->type == CD || d->type == DVD) 
    {
        QApplication::setOverrideCursor(waitCursor);
        process = new QProcess(this);
        process->addArgument("eject");
        process->addArgument(d->dDir);
        if (!process->start())
                QMessageBox::warning(0,tr("Cannot eject CD!"),tr("Cannot eject CD!"));

        else {
            while(process->isRunning())
                mainw->app->processEvents();
            process->clearArguments();
            delete (process);
            QApplication::restoreOverrideCursor();
        }
    }
#endif
#endif

  return 0;
 }

int GuiSlave::rescanEvent(void)
 {
   QString o;
   QString rfd;
   
   if(mainw->db == NULL) return 0;
   if(standON == NULL) 
    {
     QMessageBox::warning(mainw,tr("Error:"),tr("There is no selected Media on the middle list box!"),
                          tr("Ok"),0);
     return 0;
    }
   if(standON->type != HC_MEDIA)
    {
     QMessageBox::warning(mainw,tr("Error:"),tr("You can refresh ONLY a MEDIA node!"),
                          tr("Ok"),0);
     return 0;
    }
    
   o =tr("Rescan").arg(standON->getNameOf());
   rfd = QFileDialog::getExistingDirectory(NULL,mainw,"rescand",o);  
   if(rfd.isEmpty()) return 0;

   PWw *pww = new PWw(mainw);
   mainw->db->pww = pww;     
   progress(pww);
   
   panelsOFF();
   if(0 != mainw->db->addMedia(
          (char *)((const char *)rfd),"__rescanned__",-1,UNKNOWN,"system"))   
     {
      Node *d;
      o =tr("An error occured while scanning, the rescan operation cancelled: \n%1")
             .arg(mainw->db->errormsg);    
     
      d=(mainw->db->getRootNode())->child;    
      while(strcmp(d->getNameOf(),"__rescanned__") != 0) d=d->next;
      mainw->db->deleteNode(d);  
      QMessageBox::warning(mainw,tr("Error:"),o,tr("Ok"),0);
     }
    else
     {
      Node *d;
      d=(mainw->db->getRootNode())->child;    
      while(strcmp(d->getNameOf(),"__rescanned__") != 0) d=d->next;
      
      delete [] ((DBMedia *)(d->data))->name;
      ((DBMedia *)(d->data))->name =
            mstr( ((DBMedia *)(standON->data))->name );

      ((DBMedia *)(d->data))->comment = 
            mstr( ((DBMedia *)(standON->data))->comment );
	    
      delete [] ((DBMedia *)(d->data))->owner;	    
      ((DBMedia *)(d->data))->owner =
            mstr( ((DBMedia *)(standON->data))->owner );	    	    
	    
      ((DBMedia *)(d->data))->type   = ((DBMedia *)(standON->data))->type ;	    	    
      
      ((DBMedia *)(d->data))->number = ((DBMedia *)(standON->data))->number;
      
      mainw->db->deleteNode(standON);	    	          
     }    
    standON=NULL; 
    panelsON();    
   
  progress(pww);
  pww->end();
  mainw->db->pww = NULL;
  delete pww;     
    
   return 0;
 }

int GuiSlave::findEvent(void)
 {
  if(mainw->db == NULL) return 0;
  
  findDialog *d=new findDialog(mainw,"adddialog",true);
  d->exec();
  return 0;
 }

int GuiSlave::configEvent(void)
 {
  ConfigDialog *d=new ConfigDialog(mainw,"configdialog",true);
  d->exec();
  return 0;
 }

int GuiSlave::aboutEvent(void)
 {
  QMessageBox::about(mainw,tr("About CdCatalog..."),
      tr(" - = CdCatalog by Hyper = -\nVersion: %1 \nAuthor: Peter Deak (hyperr@freemail.hu)\n Copyright (C) 2003 Peter Deak (GPL)")
          .arg(VERSION));
  return 0;
 }

int GuiSlave::insertcEvent(void)
 {
  char fnc[256];
  QString fn;

  if(mainw->db == NULL) return 0;
  fn = QFileDialog::getOpenFileName(NULL,tr("CdCat databases (*.hcf )"),
                     mainw,"openfiled",tr("Insert a database file..."));
  if(fn.isEmpty())  return 0;
  
  strcpy(fnc,(const char *)fn);
 
  panelsOFF();
  
  PWw *pww = new PWw(mainw);
  mainw->db->pww = pww;     
  progress(pww);

  if(mainw->db->insertDB(fnc) != 0 )
   { // An error occured
     QMessageBox::warning(mainw,tr("Error while opening..."),
                          mainw->db->errormsg,tr("Ok"),0); 
   }

  progress(pww);
  panelsON();
  progress(pww);
  pww->end();
  mainw->db->pww = NULL;
  delete pww;
  return 0;
 }

int GuiSlave::renameEvent(void)
 {
  renamedialog *rd;
  if(mainw->db == NULL) return 0;
  if(standON == NULL) 
   {
     QMessageBox::warning(mainw,tr("Error:"),tr("There is no selected item on the middle list box!"),
                          tr("Ok"),0);
     return 0;
   }
  rd = new renamedialog(mainw,standON,this);
  rd->exec();
  cHcaption();
  updateListFromNode();
  return 0;
 }

int GuiSlave::sizeEvent(void)
 {
  char text[52];
  QString qtext;
  if(mainw->db == NULL) return 0;
  if(standON == NULL) 
   {
     QMessageBox::warning(mainw,tr("Error:"),tr("There is no selected item on the middle list box!"),
                          tr("Ok"),0);
     return 0;
   }
   
  sprintf(text,"%.2f Mb",mainw->db->getSize(standON));
  qtext = tr("The size of \"%1\" : \n %2 \n %3 file /%4 directory")
                                 .arg(standON->getNameOf())
				 .arg(text)
				 .arg(mainw->db->getCountFiles(standON))
				 .arg(mainw->db->getCountDirs (standON)); 			 
  
  QMessageBox::warning(mainw,tr("The Resoult:"),qtext,
                                           tr("Ok"),0);
  return 0;
 }

int GuiSlave::sortNuEvent(void)
 {
  if(mainw->db == NULL) return 0;
  panelsOFF();
  mainw->db->sortM(NUMBER);
  panelsON();
  return 0;
 }

int GuiSlave::sortNaEvent(void)
 {
  if(mainw->db == NULL) return 0;
  panelsOFF();
  mainw->db->sortM(NAME);
  panelsON();
  return 0;
 }

int GuiSlave::sortTyEvent(void)
 {
  if(mainw->db == NULL) return 0;
  panelsOFF();
  mainw->db->sortM(TYPE);
  panelsON();
  return 0;
 }

int GuiSlave::sortTiEvent(void)
 {
  if(mainw->db == NULL) return 0;
  panelsOFF();
  mainw->db->sortM(TIME);
  panelsON();
  return 0;
 }

int GuiSlave::helpEvent(void)
 {
  helpDialog *d=new helpDialog(mainw,"helpdialog",true);
  d->exec();
  return 0;
 }

int GuiSlave::importEvent (void)
{
    importCsv i(this);

    return 0;

}

