/****************************************************************************
                             Hyper's CD Catalog
		A multiplatform qt and xml based catalog program

 Author    : Christoph Thielecke (crissi99@gmx.de)
 License   : GPL
 Copyright : (C) 2003 Christoph Thielecke
****************************************************************************/

#include "import.h"
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdatetime.h>
#include <qprogressdialog.h>
#include "importdialog.h"

#include "mainwidget.h"
#include "dbase.h"
#include "config.h"


/*
 * import a file with folling format:
 * <media>/<path>/..../<file><seperator><size><seperator><day/month/year><space><hour>:<minute>:<second>
 * the hashmark '#' can be used as comment in csv file.
 *
 */
importCsv::importCsv(GuiSlave * parent)
{

    QString seperator=";";
    QString filename;

    ImportDialog *d=new ImportDialog(0);
    d->exec();

    if(d->OK == 1) {
        seperator = d->seperator;
        filename = d->filename;
        createdatabase = d->newdatabase->isChecked();
        correctbadstyle = d->correctbadstyle->isChecked();
        delete d;
    }

    if (!filename.isEmpty()) {

        if(parent->mainw->db == NULL)
            parent->newEvent();

        if (createdatabase){
            parent->closeEvent();
            parent->newEvent();
        }

        guislave = parent;
        db = parent->mainw->db;
        mediacount=0;
        filecount=0;
        dircount=0;
        refreshcount=0;
        int linecount=0;
        int lines=0;


        QApplication::setOverrideCursor(waitCursor);
        guislave->panelsOFF();

        QFile f(filename);

        if ( f.open( IO_ReadOnly ) ) {
            lines = f.readAll().contains( '\n' ) + 1;
            f.close();
        }

        if (f.open(IO_ReadOnly)) {	// file opened successfully

        QProgressDialog progress(0,"progressdialog",true);
        progress.setLabelText(tr("Importing csv..."));
        progress.setCancelButton(0);
        progress.setTotalSteps(lines);

	    QTextStream t(&f);	// use a text stream
	    QString medianame = "";

	    medialines = new QList < lineObject > ();
	    medialines->setAutoDelete(TRUE);	// the list owns the objects

	    while (!t.eof()) {
	        QString line;
	        QString fullpath;
	        QString path;
	        float size;
	        QDate date;
	        QTime time;
	        QString new_medianame;
	        QString datetimestring;


	        line = t.readLine();	// line of text excluding '\n'

	        if (!line.startsWith("#") && !line.isEmpty()) {

                if (correctbadstyle){
             //       QMessageBox::warning (0, "wrong", line);
                    line.replace(QString(seperator+"/"),"/");
                    line.replace(QString("/"+seperator),"/");
           //        QMessageBox::warning (0, "fine", line);
                }


		        int pathindex = line.find(seperator);
		        fullpath = (line.left(pathindex)).stripWhiteSpace();

		        int mediaindex = fullpath.find('/');
		        new_medianame = (fullpath.left(mediaindex)).stripWhiteSpace();

		        path = (fullpath.mid(mediaindex + 1, fullpath.length()));

		        int sizeindex = line.find(seperator, pathindex + 1);
		        size =  ((line.mid(pathindex + 1, sizeindex)).stripWhiteSpace()).toFloat();

		        datetimestring = (line.mid(sizeindex + 1, line.length())).stripWhiteSpace();

        		int dayindex = datetimestring.find("/");
		        int monthindex = datetimestring.find("/", dayindex + 1);
		        int yearindex = datetimestring.find(" ", monthindex + 1);
		        int hourindex = datetimestring.find(":", yearindex + 1);
		        int minuteindex = datetimestring.find(":", hourindex + 1);

		        int day = (datetimestring.mid(0, dayindex)).toInt();
		        QString day_ = (datetimestring.mid(0, dayindex));
		        int month = (datetimestring.mid(dayindex + 1, monthindex - dayindex - 1)).toInt();
		        int year = (datetimestring.mid(monthindex + 1, hourindex - monthindex - 4)).toInt();
		        bool validDate = date.setYMD(year, month, day);

		        int hour = (datetimestring.mid(yearindex + 1, minuteindex - hourindex - 1)).toInt();
		        int minute = (datetimestring.mid(hourindex + 1, minuteindex - 1 - hourindex)).toInt();;
		        int second = (datetimestring.mid(minuteindex + 1, datetimestring.length() -1)).toInt();
                QDate date(year,month,day);
                QTime time (hour,minute,second);

		        QDateTime datetime = QDateTime(date,time);

		        int fileindex = path.findRev('/');
		        QString dirpath = path.left(fileindex);
		        QString filename = path.mid(fileindex + 1, path.length());

                if (!validDate) {
                    QString msg;
		            msg += "path: " + path + "\n";
		            msg += "fullpath: " + fullpath + "\n";
		            msg += "size: " + QString().setNum(size) + "\n";
		            msg += "day: " + QString().setNum(day) + "\n";
		            msg += "month: " + QString().setNum(month) + "\n";
		            msg += "hour: " + QString().setNum(hour) + "\n";
		            msg += "minute: " + QString().setNum(minute) + "\n";
		            msg += "second: " + QString().setNum(second) + "\n";
		            msg += "year: " + QString().setNum(year) + "\n";
		            msg += "dat_: " + day_ + "\n";
		            msg += "datetimestring: " + datetimestring + "\n";
		            msg += "new_medianame: " + medianame + "\n";
		            msg+= "directory path:"+dirpath+"\n";
		            msg+="file name: "+filename+"\n";

		            QMessageBox::warning(0, "line", msg);
                }

		        if (medianame == "")
		            medianame = new_medianame;

		        if (medianame != new_medianame) {
		            //        QMessageBox::warning (0, "info", medianame);
		            addNewMedia(medianame, medialines);
		            medialines->clear();
		            medianame = new_medianame;
		            addNewItem(new_medianame, dirpath, filename, size,QDateTime(date, time));
		        } else {
		            //        QMessageBox::warning (0, "info", "new item");
		            addNewItem(medianame, dirpath, filename, size,datetime);
		        }

	        }			// valid line
            linecount++;
            progress.setProgress(linecount);
	    }
	    f.close();
	    if (!medialines->isEmpty()) {
	        addNewMedia(medianame, medialines);
	        medialines->clear();
	    }

	    delete medialines;
        (db->getRootNode())->touchDB();
        guislave->panelsON();
        QApplication::restoreOverrideCursor();

        QString msg;
        msg+=tr("Import was successful.");
        msg+="\n";
        if (mediacount < 2)
            msg+=tr("1 media")+",\n";
        else
            msg+=QString().setNum(mediacount)+" "+QString(tr("media"))+",\n";

        if (dircount < 2)
            msg+=QString(tr("1 directory:,\n"));
        else
            msg+=QString().setNum(dircount)+" "+QString(tr("directories"))+",\n";

        if (filecount < 2)
            msg+=tr("1 File")+",\n";
        else
            msg+=QString().setNum(filecount)+" "+QString(tr("files"))+"\n";

        msg+=tr("are imported.");

        QMessageBox::information(0,tr("Import successful"),msg);

        }
        else {

	        if (f.status() == IO_ReadError)
	            QMessageBox::critical(0, tr("file read error"), tr("Could not read file"));

	        else if (f.status() == IO_FatalError)
	            QMessageBox::critical(0, tr("Fatal error"), tr("Fatal error occured."));

    	    else if (f.status() == IO_OpenError)
	        QMessageBox::critical(0, tr("file open error"), tr("Could not open file"));
        }
    } // file dialog canceled
}

importCsv::~importCsv()
{

}

int
 importCsv::addNewItem(QString medianame, QString path,
		       QString filename, float size, QDateTime datetime)
{
    medialines->append(new lineObject(medianame, path, filename, size, datetime));

    // QMessageBox::critical( 0, "item", "new item!");
    return 0;

}

int importCsv::addNewMedia(QString new_medianame,
			   QList < lineObject > *medialines)
{
     // QMessageBox::critical(0, "media", new_medianame);

    if (guislave->mainw->db == NULL)
	guislave->newEvent();
    if (guislave->mainw->db == NULL)
	return 0;

    /* create new media */
    Node *env,*curr;
    curr =  db->getMediaNode((char *)((const char *)*new QString(new_medianame)));
    if(curr == NULL)
        curr = db->putMediaNode((char *)((const char *)*new QString(new_medianame)),mediacount,((char*) ((const char *)tr("importuser"))),CD,"");

    QString msg;
    lineObject *obj;

    for ( obj = medialines->first(); obj; obj = medialines->next() ) {

        env = curr;

        QString path = obj->getPath();
        path+="/";
   //     QMessageBox::warning(0, "path", path);
        int startindex=0;
        int dirindex=0;
        dirindex = path.find("/");

        while (dirindex != -1) {

            QString dir = path.mid(startindex,dirindex-startindex);
    //        QMessageBox::warning(0, "new dir", dir);

            curr = db->getDirectoryNode(env,(char *)((const char *)*new QString(dir)));
            if(curr == NULL) {
                curr = db->putDirectoryNode(env,(char *)((const char *)*new QString(dir)),getTime(obj->getDateTime()),"");
            dircount++;

           }

           startindex=dirindex+1;
           dirindex = path.find("/",dirindex+1);

           env = curr;

        }
        dirindex=-1;

        uint size = (uint)obj->getSize();
	    float s;
	    int   st;

	    if(size > (uint)(1024*1024*1024*2))
	    {
	        s  = (double)size / (double)(1024*1024*1024);
	        st = GBYTE;
	    }
	    else if(size > (uint)(1024*1024))
	    {
	        s  = (double)size / (double)(1024*1024);
	        st = MBYTE;
	    }
	    else if(size > (uint)1024)
	    {
	        s  = (double)size / (double)1024;
	        st = KBYTE;
	    }
	    else
	    {
	        s  = size;
	        st = BYTE;
	    }

        env = curr;
        curr = db->getFileNode(env,((char *) ((const char*)*new QString(obj->getFileName()))));
        if(curr == NULL)
            curr = db->putFileNode(env,((char *)((const char *)*new QString(obj->getFileName()))),getTime(obj->getDateTime()),"",st,s);

         curr =  db->getMediaNode((char *)((const char *)*new QString(new_medianame)));

         filecount++;
    } // over all items in list

    mediacount++;
    refreshcount++;
    if(refreshcount == 10) {
        guislave->mainw->app->processEvents();
        refreshcount=0;
    }

    return 0;
}

lineObject::lineObject(QString medianame, QString path, QString filename,
		       float size, QDateTime datetime)
{
    this->medianame = medianame;
    this->path = path;
    this->filename = filename;
    this->size = size;
    this->datetime =datetime;
}

lineObject::~lineObject()
{

}

QString lineObject::getMediaName()
{
    return medianame;
}

QString lineObject::getPath()
{
    return path;
}

QString lineObject::getFileName()
{
    return filename;
}

float lineObject::getSize()
{
    return size;
}

QDateTime lineObject::getDateTime()
{
    return datetime;
}


