/****************************************************************************
                             Hyper's CD Catalog 
		A multiplatform qt and xml based catalog program
  
 Author    : Peter Deak (hyperr@freemail.hu)
 License   : GPL
 Copyright : (C) 2003 Peter Deak
****************************************************************************/

#include <stdio.h>
#include <ctype.h>

#include <expat.h>
#include <zlib.h>
#include <qstring.h>

#include "wdbfile.h"
#include "adddialog.h"
#include "recode.h"
#include "dbase.h"
#include "cdcat.h"



//#define XML_ENCODING   "UTF-8"
#define XML_ENCODING   "ISO-8859-1"
#define BUFFSIZE	8192

char *out1=NULL;
char *out2=NULL;
char *out3=NULL;
char *out4=NULL;
char *out5=NULL;

/*************************************************************************/

int getTypeFS(const char *str)
 {
  if(!strcmp("CD"          ,str)) return CD;  
  if(!strcmp("DVD"         ,str)) return DVD;
  if(!strcmp("HardDisc"    ,str)) return HARDDISC; 
  if(!strcmp("floppy"      ,str)) return FLOPPY;
  if(!strcmp("NetworkPlace",str)) return NETPLACE;  
  if(!strcmp("flashdrive"  ,str)) return FLASHDRV;
  if(!strcmp("other"       ,str)) return OTHERD;   
  return UNKNOWN;   
 }

const char *getMType(int t)
{
 switch(t)
  {
   case UNKNOWN : return "unknown";
   case CD      : return "CD";
   case DVD     : return "DVD";
   case HARDDISC: return "HardDisc";
   case FLOPPY  : return "floppy";
   case NETPLACE: return "NetworkPlace";
   case FLASHDRV: return "flashdrive";
   case OTHERD  : return "other";
  }
 return NULL;  
}

float getSizeFS(const char *str)
 {
  float  r;
  char   s[10];
  sscanf(str,"%f %s",&r,s);
  return r;
 }
 
int   getSizetFS(const char *str)
 {
  float  r;
  char   s[10];
  sscanf(str,"%f %s",&r,s);
  if(!strcmp(s,"byte")) return BYTE;
  if(!strcmp(s,"Kb")) return KBYTE;
  if(!strcmp(s,"Mb")) return MBYTE;
  if(!strcmp(s,"Gb")) return GBYTE;
  return 0;
 }

const char *getSType(int t)
{
 switch(t)
  {
   case BYTE : return " byte";
   case KBYTE: return " Kb";
   case MBYTE: return " Mb";
   case GBYTE: return " Gb";
  }
 return NULL; 
}

/*****************************************************************************
 WRITING FILE:
*****************************************************************************/
char span[256];
const char *FileWriter::spg(int spn)
 {
  int  i;
  
  if(!nicef) return "";

  for(i=0;i<254 && i<(spn*2);i++) 
     span[i]=' ';
  span[i]='\0';
  return span;
 }

FileWriter::FileWriter(gzFile ff,bool nicefp)
 {
  nicef = nicefp;
  level = 0;
  f = ff;
 }


void FileWriter::commentWriter(char *c)
 {
  if(c == NULL) 
       return;
  level++;       
  gzprintf(f,"%s<comment>",spg(level));
  gzprintf(f,"%s",recode(c,&out1));
  gzprintf(f,"%s</comment>\n",spg(level));
  level--;
 }

int  FileWriter::writeDown(Node *source)
 {
  int i=0;
  switch(source->type)
   {
    case 0: return 1;
    case 1: writeHeader(source); 
            i=writeCatalog(source);
    break;
    case 2: i=writeMedia(source);    
    break;
    case 3: i=writeDirectory(source);    
    break;
    case 4: i=writeFile(source);    
    break;
    case 5: i=writeMp3Tag(source);    
    break;
   }  
   return i;
  }

int  FileWriter::writeHeader(Node *source)
 {
  level = 0;
  progress(pww);
  gzprintf(f,"<?xml version=\"1.0\" encoding=\"%s\"?>",XML_ENCODING);
  gzprintf(f,"\n<!DOCTYPE HyperCDCatalogFile>\n\n");
  gzprintf(f,"\n<!-- CD Catalog Database file, generated by CDCAT (author: Pe'ter Dea'k) -->\n");
  return 0;
 }
 
int  FileWriter::writeCatalog(Node *source)
 {
  progress(pww);
  gzprintf(f,"<catalog name=\"%s\" owner=\"%s\" time=\"%s\">\n",
            recode(((DBCatalog *)(source->data))->name,&out1),
	    recode(((DBCatalog *)(source->data))->owner,&out2),
	    recode(((DBCatalog *)(source->data))->modification,&out3) );

  commentWriter(((DBCatalog *)(source->data))->comment);
  level++;
  if(source->child != NULL) writeDown(source->child);
  level--;
  gzprintf(f,"</catalog>\n");      	    
  if(source->next  != NULL) writeDown(source->next);

  return 0;
 }
 
int  FileWriter::writeMedia(Node *source)
 {
  progress(pww);
  gzprintf(f,"%s<media name=\"%s\" number=\"%d\" owner=\"%s\" type=\"%s\" time=\"%s\">\n",
            spg(level),
            recode(((DBMedia *)(source->data))->name,&out1),
	    ((DBMedia *)(source->data))->number,
	    recode(((DBMedia *)(source->data))->owner,&out2),
	    getMType(((DBMedia *)(source->data))->type),
	    recode(((DBMedia *)(source->data))->modification,&out3));
  
  commentWriter(((DBMedia *)(source->data))->comment);
  level++;
  if(source->child != NULL) writeDown(source->child);
  level--;
  gzprintf(f,"%s</media>\n",spg(level));      	    
  if(source->next  != NULL) writeDown(source->next);

  return 0;
 }
 
int  FileWriter::writeDirectory(Node *source)
 {
  progress(pww);
  gzprintf(f,"%s<directory name=\"%s\" time=\"%s\">\n",
            spg(level),
            recode(((DBDirectory *)(source->data))->name,&out1),
	    recode(((DBDirectory *)(source->data))->modification,&out3));
  
  commentWriter(((DBDirectory *)(source->data))->comment);
 
  level++;
  if(source->child != NULL) writeDown(source->child);
  level--;
  gzprintf(f,"%s</directory>\n",spg(level));      	    
  if(source->next  != NULL) writeDown(source->next);

  return 0; 
 }
 
int  FileWriter::writeFile(Node *source)
 {
  gzprintf(f,"%s<file name=\"%s\" size=\"%.2f%s\" time=\"%s\">\n",
            spg(level),
            recode(((DBFile *)(source->data))->name,&out1),
	    ((DBFile *)(source->data))->size,
	    getSType((((DBFile *)(source->data))->sizeType)),
	    recode(((DBFile *)(source->data))->modification,&out3));
  commentWriter(((DBFile *)(source->data))->comment);
  level++;
  if(source->child != NULL) 
           writeDown(source->child);
  if(((DBFile *)(source->data))->prop != NULL) 
           writeDown(((DBFile *)(source->data))->prop);
  level--;
  gzprintf(f,"%s</file>\n",spg(level));      	    
  if(source->next  != NULL) writeDown(source->next);
  return 0; 
 }

int  FileWriter::writeMp3Tag(Node *source)
 {
  gzprintf(f,"%s<mp3tag artist=\"%s\" title=\"%s\" album=\"%s\"  year=\"%s\">",
            spg(level),
            recode(((DBMp3Tag *)(source->data))->artist,&out1),
            recode(((DBMp3Tag *)(source->data))->title,&out2),
            recode(((DBMp3Tag *)(source->data))->album,&out3),
            recode(((DBMp3Tag *)(source->data))->year,&out4));
  gzprintf(f,"%s",recode(((DBMp3Tag *)(source->data))->comment,&out5));      	    
  gzprintf(f,"%s</mp3tag>\n",spg(level));      	    
  if(source->next  != NULL) writeDown(source->next);
  return 0;
 }

/*****************************************************************************
 READING FILE:
*****************************************************************************/

float FileReader::getFloat(const char **from,char *what,char *err)
 {
   float r;
   int i;

   if(from == NULL)
    {
     errormsg = QString(err);
     error = 1;
     return 0;
    }
   for(i=0;;i+=2)
    {
        if(from[i] == NULL)
          {
	    errormsg = QString("%1,I can't find %2 attribute").arg(err).arg(what);  
            error  = 1;
	    return   0;
          }
        if(!strcmp(from[i],what))
          {
            if(from[i+1] == NULL)
              {
	       errormsg = QString(err);
               error  = 1;	       
	       return 0;
              }
            if(1 != sscanf(from[i+1],"%f",& r))
              {
	        errormsg = QString("%1,I can't understand %2 attribute").arg(err).arg(what);  
		
                error  = 1;
                return   0;
             }
            return r;
          }
    }
  return 0;
 }

char * FileReader::getStr(const char **from,char *what,char *err)
 {
   int i;

   if(from == NULL)
    {
     errormsg = QString(err);
     error=1;
     return NULL;
    }
   for(i=0;;i+=2)
    {
        if(from[i] == NULL)
          {
	    errormsg = QString("%1,I can't find %2 attribute").arg(err).arg(what);  
            
	    error  = 1;
	    return   NULL;
          }
        if(!strcmp(from[i],what))
          {
            if(from[i+1] == NULL)
              {
	       errormsg = QString(err);
               error = 1;
               return NULL;
              }
	    return ((char *)(from[i+1]));  
          }
    }
  return NULL;
 }
 
int FileReader::isthere(const char **from,char *what)
 {
   int i;

   if(from == NULL)
     return 0;
   for(i=0;;i+=2)
    {
        if(from[i] == NULL)
          return 0;
        if(!strcmp(from[i],what))
          return 1;
    }
  return 0;
 }

#define FREA ((FileReader *)(data))

DBMp3Tag *tmp_tagp=NULL;

static void start(void *data, const char *el, const char **attr) 
 {
  if(!strcmp(el,"catalog"))
   {
    if(FREA->insert) return;
    
    ((DBCatalog *)((FREA->sp)->data)) ->name = 
         mstr(FREA->getStr(attr,"name","Error while parsing \"catalog\" node")); 

    ((DBCatalog *)((FREA->sp)->data)) ->owner= 
         mstr(FREA->getStr(attr,"owner","Error while parsing \"catalog\" node")); 
    strcpy(
          ((DBCatalog *)((FREA->sp)->data)) ->modification, 
          FREA->getStr(attr,"time","Error while parsing \"catalog\" node")); 
   }

 if(!strcmp(el,"media"))
   {
    Node *tt = FREA->sp->child;
    
    if(tt == NULL) FREA->sp->child = tt = new Node(HC_MEDIA,FREA->sp);
    else
     {
      while(tt->next != NULL) tt = tt->next;
      tt->next =  new Node(HC_MEDIA,FREA->sp);
      tt=tt->next; 
     }    
     
    if(FREA->insert)
     {
       int i,newnum,ser=0;
       Node *ch = tt->parent->child;
       char newname[512];
       
       strcpy(newname,FREA->getStr  (attr,"name"  ,"Error while parsing \"media\" node"));
       newnum = (int)FREA->getFloat (attr,"number","Error while parsing \"media\" node");

       /*make the number ientical*/           
       for(i=1;i==1;)
        {
	 i=0;
         for(ch = tt->parent->child;ch != NULL; ch = ch->next)
          {
	   if(ch->data != NULL)
	     if(((DBMedia *)(ch->data))->number ==  newnum )
	       i=1; 
	  }
	 if(i) newnum++; 
	}  

       /*make the name ientical*/           
       for(i=1;i==1;)
        {
	 i=0;
         for(ch = tt->parent->child;ch != NULL; ch = ch->next)
          {
	   if(ch->data != NULL)
	     if(strcmp(((DBMedia *)(ch->data))->name ,  newname)==0)
	       i=1; 
	  }
	 if(i) 
	  { 
	    char tmp[512];
	    ser++;
	    strcpy(tmp,newname);
	    sprintf(newname,"%s#%d",tmp,ser);
	  } 
	}  

	
        /*Fill data part:*/
        tt->data = (void *) new DBMedia(
                mstr(newname),newnum,
                mstr(FREA->getStr  (attr,"owner" ,"Error while parsing \"media\" node")),
		getTypeFS(FREA->getStr  (attr,"type"  ,"Error while parsing \"media\" node")),		
		NULL,
   	        FREA->getStr  (attr,"time"  ,"Error while parsing \"media\" node"));
     	
     } 
    else
     {
      /*Fill data part:*/
      tt->data = (void *) new DBMedia(
                mstr(FREA->getStr  (attr,"name"  ,"Error while parsing \"media\" node")),
		(int)FREA->getFloat(attr,"number","Error while parsing \"media\" node"),
                mstr(FREA->getStr  (attr,"owner" ,"Error while parsing \"media\" node")),
		getTypeFS(FREA->getStr  (attr,"type"  ,"Error while parsing \"media\" node")),		
		NULL,
   	        FREA->getStr  (attr,"time"  ,"Error while parsing \"media\" node"));
     }
    /*Make this node to the actual node:*/
    FREA->sp = tt;
   }
 if(!strcmp(el,"directory"))
   {
    Node *tt = FREA->sp->child;
    
    if(tt == NULL) FREA->sp->child = tt = new Node(HC_DIRECTORY,FREA->sp);
    else
     {
      while(tt->next != NULL) tt = tt->next;
      tt->next =  new Node(HC_DIRECTORY,FREA->sp);
      tt=tt->next; 
     }    
    /*Fill data part:*/
    tt->data = (void *) new DBDirectory(
                mstr(FREA->getStr  (attr,"name"  ,"Error while parsing \"directory\" node")),
                     FREA->getStr  (attr,"time" ,"Error while parsing \"directory\" node"),
		NULL);
    /*Make this node to the actual node:*/
    FREA->sp = tt;   
   }
 if(!strcmp(el,"file"))
   {
   
    Node *tt = FREA->sp->child;
    
    if(tt == NULL) FREA->sp->child = tt = new Node(HC_FILE,FREA->sp);
    else
     {
      while(tt->next != NULL) tt = tt->next;
      tt->next =  new Node(HC_FILE,FREA->sp);
      tt=tt->next; 
     }    
    /*Fill data part:*/
    
    tt->data = (void *) new DBFile(
                mstr(FREA->getStr  (attr,"name"  ,"Error while parsing \"file\" node")),
	        FREA->getStr  (attr,"time"  ,"Error while parsing \"file\" node"),
		NULL,
		getSizeFS (FREA->getStr(attr,"size","Error while parsing \"file\" node")),
		getSizetFS(FREA->getStr(attr,"size","Error while parsing \"file\" node")));
    
    /*Make this node to the actual node:*/
    FREA->sp = tt;   
   
   }
 if(!strcmp(el,"mp3tag"))
   {
    Node *tt =  ((DBFile *)(FREA->sp->data))->prop;
    if(tt == NULL) ((DBFile *)(FREA->sp->data))->prop = tt = new Node(HC_MP3TAG,FREA->sp);
    else
     {
      while(tt->next != NULL) tt = tt->next;
      tt->next =  new Node(HC_MP3TAG,FREA->sp);
      tt=tt->next; 
     }    
    /*Fill data part:*/
    
    tmp_tagp = new DBMp3Tag(
                FREA->getStr(attr,"artist"  ,"Error while parsing \"mp3tag\" node"),
	        FREA->getStr(attr,"title"   ,"Error while parsing \"mp3tag\" node"),
		"no comment",
                FREA->getStr(attr,"album"  ,"Error while parsing \"mp3tag\" node"),
	        FREA->getStr(attr,"year"   ,"Error while parsing \"mp3tag\" node"));
		
    tt->data = (void *)tmp_tagp;		
    /*I don't make this node to the actual node because this won't be parent.*/
   }

 if(!strcmp(el,"comment"))
   {
    /*nothing*/
   }
 }  
/*********************************************************************/
static void end(void *data, const char *el) 
 {
  if(!strcmp(el,"catalog"))
   {
    //End parsing ! 
   }

 if(!strcmp(el,"media"))
   {
    /*Restore the parent node tho the actual node:*/
    FREA->sp = FREA->sp->parent;     
   }
 if(!strcmp(el,"directory"))
   {
    /*Restore the parent node tho the actual node:*/
    FREA->sp = FREA->sp->parent;     
   }
 if(!strcmp(el,"file"))
   {
    /*Restore the parent node tho the actual node:*/
    FREA->sp = FREA->sp->parent;     
   }
 if(!strcmp(el,"mp3tag"))
   {
    if(strlen(FREA->dataBuffer)>31) /*Cut the string if necessary*/
        FREA->dataBuffer[31] = '\0';
    strcpy(tmp_tagp->comment,FREA->dataBuffer);		
    tmp_tagp = NULL;
   }

 if(!strcmp(el,"comment"))
   {
   switch(FREA->sp->type)
     {
      case HC_CATALOG  : 

          ((DBCatalog    *)(FREA->sp->data)) -> comment = mstr(FREA->dataBuffer);
         break;
      case HC_MEDIA    : 
          ((DBMedia      *)(FREA->sp->data)) -> comment = mstr(FREA->dataBuffer);
         break;
      case HC_DIRECTORY: 
          ((DBDirectory *)(FREA->sp->data)) -> comment = mstr(FREA->dataBuffer);
         break;
      case HC_FILE     : 
          ((DBFile      *)(FREA->sp->data)) -> comment = mstr(FREA->dataBuffer);
         break;
      case HC_MP3TAG:
	  FREA->errormsg = QString("It isnt allowed comment node in mp3tag node.");
          FREA->error = 1;
         break;	 
     }
   }   
 }

void getCharDataFromXML(void *data,const char *s,int len)
 {
  int i;
  for(i=0;i<len && i<1023;i++)
     FREA->dataBuffer[i] = s[i];
  FREA->dataBuffer[i] = '\0';       
 }
 

int FileReader::readFrom(Node *source) 
 {
  char *Buff = new char[BUFFSIZE];
  XML_Parser p = XML_ParserCreate(NULL);
 
  error = 0;
  sp    = source;

  dataBuffer = new char[1024];
  if (p == NULL)
   {
    errormsg = QString("Couldn't allocate memory for parser");
    delete [] Buff;
    delete [] dataBuffer;
    return 1;
   }

  XML_SetUserData(p,this);
  XML_SetElementHandler(p, start, end);
  XML_SetCharacterDataHandler(p,getCharDataFromXML);
  for (;;) 
  {
    int done;
    int len;

    progress(pww);
    
    len = gzread(f,Buff,BUFFSIZE);
   if (len == -1)
     {
      errormsg = QString("Read error");
      delete [] Buff;
      delete [] dataBuffer;
      return 1;
     }

    done = gzeof(f);

    if (! XML_Parse(p, Buff, len, done)) 
     {
      errormsg = QString("Parse error at line %1:\n%2\n")
                     .arg(XML_GetCurrentLineNumber(p))
		     .arg(XML_ErrorString(XML_GetErrorCode(p)));
      
      delete [] Buff;
      delete [] dataBuffer;
      return 1;
     }

    if(done || error)
     {
      delete [] Buff;
      delete [] dataBuffer;
      return error;
     }
  }
}


  

